@@ -16,16 +16,22 @@
 # License along with SETools.  If not, see
 # <http://www.gnu.org/licenses/>.
 #
-from collections import namedtuple
+from typing import NamedTuple
+
+from ..policyrep import Context, Portcon
 
 from .context import ContextWrapper
 from .descriptors import DiffResultDescriptor
 from .difference import Difference, Wrapper
 
 
-modified_portcon_record = namedtuple("modified_portcon", ["rule",
-                                                          "added_context",
-                                                          "removed_context"])
+class ModifiedPortcon(NamedTuple):
+
+    """Difference details for a modified portcon."""
+
+    rule: Portcon
+    added_context: Context
+    removed_context: Context
 
 
 class PortconsDifference(Difference):
@@ -36,7 +42,7 @@ class PortconsDifference(Difference):
     removed_portcons = DiffResultDescriptor("diff_portcons")
     modified_portcons = DiffResultDescriptor("diff_portcons")
 
-    def diff_portcons(self):
+    def diff_portcons(self) -> None:
         """Generate the difference in portcons between the policies."""
 
         self.log.info("Generating portcon differences from {0.left_policy} to {0.right_policy}".
@@ -52,14 +58,14 @@ class PortconsDifference(Difference):
             # Criteria for modified portcons
             # 1. change to context
             if ContextWrapper(left_portcon.context) != ContextWrapper(right_portcon.context):
-                self.modified_portcons.append(modified_portcon_record(left_portcon,
-                                                                      right_portcon.context,
-                                                                      left_portcon.context))
+                self.modified_portcons.append(ModifiedPortcon(left_portcon,
+                                                              right_portcon.context,
+                                                              left_portcon.context))
 
     #
     # Internal functions
     #
-    def _reset_diff(self):
+    def _reset_diff(self) -> None:
         """Reset diff results on policy changes."""
         self.log.debug("Resetting portcon differences")
         self.added_portcons = None
@@ -67,13 +73,14 @@ class PortconsDifference(Difference):
         self.modified_portcons = None
 
 
-class PortconWrapper(Wrapper):
+# Pylint bug: https://github.com/PyCQA/pylint/issues/2822
+class PortconWrapper(Wrapper[Portcon]):  # pylint: disable=unsubscriptable-object
 
     """Wrap portcon statements for diff purposes."""
 
     __slots__ = ("protocol", "low", "high")
 
-    def __init__(self, ocon):
+    def __init__(self, ocon: Portcon) -> None:
         self.origin = ocon
         self.protocol = ocon.protocol
         self.low, self.high = ocon.ports
