@@ -17,8 +17,14 @@
 # <http://www.gnu.org/licenses/>.
 #
 
+from contextlib import suppress
+from typing import Iterable, List, Optional, Tuple
 
-def match_regex(obj, criteria, regex):
+from .exception import InvalidPermission, NoCommon
+from .policyrep import Level, ObjClass, SELinuxPolicy
+
+
+def match_regex(obj, criteria, regex: bool) -> bool:
     """
     Match the object with optional regular expression.
 
@@ -34,7 +40,7 @@ def match_regex(obj, criteria, regex):
         return obj == criteria
 
 
-def match_set(obj, criteria, equal):
+def match_set(obj, criteria, equal: bool) -> bool:
     """
     Match the object (a set) with optional set equality.
 
@@ -51,7 +57,7 @@ def match_set(obj, criteria, equal):
         return bool(obj.intersection(criteria))
 
 
-def match_in_set(obj, criteria, regex):
+def match_in_set(obj, criteria, regex: bool) -> bool:
     """
     Match if the criteria is in the list, with optional
     regular expression matching.
@@ -63,12 +69,12 @@ def match_in_set(obj, criteria, regex):
     """
 
     if regex:
-        return [m for m in obj if criteria.search(str(m))]
+        return bool([m for m in obj if criteria.search(str(m))])
     else:
         return criteria in obj
 
 
-def match_indirect_regex(obj, criteria, indirect, regex):
+def match_indirect_regex(obj, criteria, indirect: bool, regex: bool) -> bool:
     """
     Match the object with optional regular expression and indirection.
 
@@ -82,14 +88,14 @@ def match_indirect_regex(obj, criteria, indirect, regex):
 
     if indirect:
         if regex:
-            return [o for o in obj.expand() if criteria.search(str(o))]
+            return bool([o for o in obj.expand() if criteria.search(str(o))])
         else:
-            return set(criteria.expand()).intersection(obj.expand())
+            return bool(set(criteria.expand()).intersection(obj.expand()))
     else:
         return match_regex(obj, criteria, regex)
 
 
-def match_regex_or_set(obj, criteria, equal, regex):
+def match_regex_or_set(obj, criteria, equal: bool, regex: bool) -> bool:
     """
     Match the object (a set) with either set comparisons
     (equality or intersection) or by regex matching of the
@@ -106,12 +112,12 @@ def match_regex_or_set(obj, criteria, equal, regex):
     """
 
     if regex:
-        return [m for m in obj if criteria.search(str(m))]
+        return bool([m for m in obj if criteria.search(str(m))])
     else:
         return match_set(obj, set(criteria), equal)
 
 
-def match_range(obj, criteria, subset, overlap, superset, proper):
+def match_range(obj, criteria, subset: bool, overlap: bool, superset: bool, proper: bool) -> bool:
     """
     Match ranges of objects.
 
@@ -144,7 +150,7 @@ def match_range(obj, criteria, subset, overlap, superset, proper):
         return criteria.low == obj.low and obj.high == criteria.high
 
 
-def match_level(obj, criteria, dom, domby, incomp):
+def match_level(obj: Level, criteria: Level, dom: bool, domby: bool, incomp: bool) -> bool:
     """
     Match the an MLS level.
 
@@ -163,3 +169,86 @@ def match_level(obj, criteria, dom, domby, incomp):
         return (criteria ^ obj)
     else:
         return (criteria == obj)
+
+
+def validate_perms_any(perms: Iterable[str], tclass: Optional[Iterable[ObjClass]] = None,
+                       policy: Optional[SELinuxPolicy] = None) -> None:
+    """
+    Validate that each permission is valid for at least one
+    of specified object classes.  If no classes are specified,
+    then all classes in the policy are checked.
+
+    A tclass or policy must be specified.
+
+    Parameters:
+    perms       A container of str permission names.
+
+    Keyword Parameters.
+    tclass      An iterable of 1 or more ObjClass.
+    policy      A SELinuxPolicy
+
+    Exceptions:
+    ValueError          Invalid parameter.
+    InvalidPermission   One or more permissions is invalid.
+
+    Return:     None
+    """
+
+    if not perms:
+        raise ValueError("No permissions specified.")
+
+    if tclass:
+        # make local mutable set
+        selected_classes = set(c for c in tclass)
+    elif policy:
+        selected_classes = set(policy.classes())
+    else:
+        raise ValueError("No object class(es) or policy specified.")
+
+    invalid = set(p for p in perms)
+    for c in selected_classes:
+        invalid -= c.perms
+
+        with suppress(NoCommon):
+            invalid -= c.common.perms
+
+        if not invalid:
+            break
+    else:
+        if tclass:
+            raise InvalidPermission(
+                "Permission(s) do not exist in the specified classes: {}"
+                .format(", ".join(invalid)))
+        else:
+            raise InvalidPermission(
+                "Permission(s) do not exist any class: {}"
+                .format(", ".join(invalid)))
+
+
+def xperm_str_to_tuple_ranges(perms: str, separator: str = ",") -> List[Tuple[int, int]]:
+    """
+    Create a extended permission list of ranges from a string representation of ranges.
+    This does not do any checking for out-of-range values.
+
+    Parameters:
+    perms       A string representation of integer extended permissions, such as
+                "0x08,0x30-0x40,0x55,0x60-0x65"
+
+    Keyword Parameters:
+    separator   The separator between permissions/permission ranges.
+                Default is ","
+
+    Return:     List[Tuple[int, int]] equivalent of the permissions.
+    """
+
+    xperms: List[Tuple[int, int]] = []
+    for item in perms.split(separator):
+        rng = item.split("-")
+        if len(rng) == 2:
+            xperms.append((int(rng[0], base=16), int(rng[1], base=16)))
+        elif len(rng) == 1:
+            xperms.append((int(rng[0], base=16), int(rng[0], base=16)))
+        else:
+            raise ValueError("Unable to parse \"{}\" for xperms.".format(item))
+
+    return xperms
