@@ -19,6 +19,7 @@
 #
 
 import logging
+from contextlib import suppress
 
 from PyQt5.QtCore import Qt, QSortFilterProxyModel, QStringListModel, QThread
 from PyQt5.QtGui import QPalette, QTextCursor
@@ -28,16 +29,20 @@ from setools import DefaultQuery, DefaultValue, DefaultRangeValue
 from ..logtosignal import LogHandlerToSignal
 from ..models import SEToolsListModel, invert_list_selection
 from ..defaultmodel import DefaultTableModel
-from .analysistab import AnalysisTab
+from .analysistab import AnalysisSection, AnalysisTab
 from .queryupdater import QueryResultsUpdater
 from .workspace import load_checkboxes, load_comboboxes, load_listviews, load_textedits, \
-                       save_checkboxes, save_comboboxes, save_listviews, save_textedits
+    save_checkboxes, save_comboboxes, save_listviews, save_textedits
 
 
 class DefaultQueryTab(AnalysisTab):
 
     """Default browser and query tab."""
 
+    section = AnalysisSection.Other
+    tab_title = "Defaults"
+    mlsonly = False
+
     def __init__(self, parent, policy, perm_map):
         super(DefaultQueryTab, self).__init__(parent)
         self.log = logging.getLogger(__name__)
@@ -46,8 +51,10 @@ class DefaultQueryTab(AnalysisTab):
         self.setupUi()
 
     def __del__(self):
-        self.thread.quit()
-        self.thread.wait(5000)
+        with suppress(RuntimeError):
+            self.thread.quit()
+            self.thread.wait(5000)
+
         logging.getLogger("setools.defaultquery").removeHandler(self.handler)
 
     def setupUi(self):
