@@ -18,6 +18,7 @@
 #
 
 import logging
+from contextlib import suppress
 
 from PyQt5.QtCore import pyqtSignal, Qt, QStringListModel, QThread
 from PyQt5.QtGui import QPalette, QTextCursor
@@ -26,7 +27,7 @@ from PyQt5.QtWidgets import QCompleter, QHeaderView, QMessageBox, QProgressDialo
 from setools import DomainTransitionAnalysis
 
 from ..logtosignal import LogHandlerToSignal
-from .analysistab import AnalysisTab
+from .analysistab import AnalysisSection, AnalysisTab
 from .excludetypes import ExcludeTypes
 from .exception import TabFieldError
 from .workspace import load_checkboxes, load_spinboxes, load_lineedits, load_textedits, \
@@ -37,6 +38,10 @@ class DomainTransitionAnalysisTab(AnalysisTab):
 
     """A domain transition analysis tab."""
 
+    section = AnalysisSection.Analysis
+    tab_title = "Domain Transition Analysis"
+    mlsonly = False
+
     def __init__(self, parent, policy, perm_map):
         super(DomainTransitionAnalysisTab, self).__init__(parent)
         self.log = logging.getLogger(__name__)
@@ -47,8 +52,10 @@ class DomainTransitionAnalysisTab(AnalysisTab):
         self.setupUi()
 
     def __del__(self):
-        self.thread.quit()
-        self.thread.wait(5000)
+        with suppress(RuntimeError):
+            self.thread.quit()
+            self.thread.wait(5000)
+
         logging.getLogger("setools.dta").removeHandler(self.handler)
 
     def setupUi(self):
