@@ -30,7 +30,7 @@ from setools import InfoFlowAnalysis
 from setools.exception import UnmappedClass, UnmappedPermission
 
 from ..logtosignal import LogHandlerToSignal
-from .analysistab import AnalysisTab
+from .analysistab import AnalysisSection, AnalysisTab
 from .exception import TabFieldError
 from .excludetypes import ExcludeTypes
 from .permmapedit import PermissionMapEditor
@@ -42,6 +42,10 @@ class InfoFlowAnalysisTab(AnalysisTab):
 
     """An information flow analysis tab."""
 
+    section = AnalysisSection.Analysis
+    tab_title = "Information Flow Analysis"
+    mlsonly = False
+
     @property
     def perm_map(self):
         return self.query.perm_map
@@ -72,8 +76,10 @@ class InfoFlowAnalysisTab(AnalysisTab):
         self.setupUi()
 
     def __del__(self):
-        self.thread.quit()
-        self.thread.wait(5000)
+        with suppress(RuntimeError):
+            self.thread.quit()
+            self.thread.wait(5000)
+
         logging.getLogger("setools.infoflow").removeHandler(self.handler)
 
     def setupUi(self):
