@@ -18,6 +18,7 @@
 #
 
 import logging
+from contextlib import suppress
 
 from PyQt5.QtCore import Qt, QSortFilterProxyModel, QStringListModel, QThread
 from PyQt5.QtGui import QPalette, QTextCursor
@@ -26,7 +27,7 @@ from setools import NetifconQuery
 
 from ..logtosignal import LogHandlerToSignal
 from ..netifconmodel import NetifconTableModel
-from .analysistab import AnalysisTab
+from .analysistab import AnalysisSection, AnalysisTab
 from .exception import TabFieldError
 from .queryupdater import QueryResultsUpdater
 from .workspace import load_checkboxes, load_lineedits, load_textedits, \
@@ -37,6 +38,10 @@ class NetifconQueryTab(AnalysisTab):
 
     """A netifcon query."""
 
+    section = AnalysisSection.Labeling
+    tab_title = "Network Interface Contexts"
+    mlsonly = False
+
     def __init__(self, parent, policy, perm_map):
         super(NetifconQueryTab, self).__init__(parent)
         self.log = logging.getLogger(__name__)
@@ -45,8 +50,10 @@ class NetifconQueryTab(AnalysisTab):
         self.setupUi()
 
     def __del__(self):
-        self.thread.quit()
-        self.thread.wait(5000)
+        with suppress(RuntimeError):
+            self.thread.quit()
+            self.thread.wait(5000)
+
         logging.getLogger("setools.netifconquery").removeHandler(self.handler)
 
     def setupUi(self):
