@@ -19,6 +19,7 @@
 #
 import sys
 import logging
+from contextlib import suppress
 
 from PyQt5.QtCore import Qt, QSortFilterProxyModel, QStringListModel, QThread
 from PyQt5.QtGui import QPalette, QTextCursor
@@ -27,7 +28,7 @@ from setools import NodeconQuery, NodeconIPVersion
 
 from ..logtosignal import LogHandlerToSignal
 from ..nodeconmodel import NodeconTableModel
-from .analysistab import AnalysisTab
+from .analysistab import AnalysisSection, AnalysisTab
 from .exception import TabFieldError
 from .queryupdater import QueryResultsUpdater
 from .workspace import load_checkboxes, load_lineedits, load_textedits, load_comboboxes, \
@@ -38,6 +39,10 @@ class NodeconQueryTab(AnalysisTab):
 
     """An nodecon query."""
 
+    section = AnalysisSection.Labeling
+    tab_title = "Network Node Contexts"
+    mlsonly = False
+
     def __init__(self, parent, policy, perm_map):
         super(NodeconQueryTab, self).__init__(parent)
         self.log = logging.getLogger(__name__)
@@ -46,8 +51,10 @@ class NodeconQueryTab(AnalysisTab):
         self.setupUi()
 
     def __del__(self):
-        self.thread.quit()
-        self.thread.wait(5000)
+        with suppress(RuntimeError):
+            self.thread.quit()
+            self.thread.wait(5000)
+
         logging.getLogger("setools.nodeconquery").removeHandler(self.handler)
 
     def setupUi(self):
