@@ -18,6 +18,7 @@
 #
 
 import logging
+from contextlib import suppress
 
 from PyQt5.QtCore import Qt, QSortFilterProxyModel, QStringListModel, QThread
 from PyQt5.QtGui import QPalette, QTextCursor
@@ -27,17 +28,21 @@ from setools import TypeQuery
 from ..logtosignal import LogHandlerToSignal
 from ..models import SEToolsListModel, invert_list_selection
 from ..typemodel import TypeTableModel, type_detail
-from .analysistab import AnalysisTab
+from .analysistab import AnalysisSection, AnalysisTab
 from .exception import TabFieldError
 from .queryupdater import QueryResultsUpdater
 from .workspace import load_checkboxes, load_lineedits, load_listviews, load_textedits, \
-                       save_checkboxes, save_lineedits, save_listviews, save_textedits
+    save_checkboxes, save_lineedits, save_listviews, save_textedits
 
 
 class TypeQueryTab(AnalysisTab):
 
     """Type browser and query tab."""
 
+    section = AnalysisSection.Components
+    tab_title = "Types"
+    mlsonly = False
+
     def __init__(self, parent, policy, perm_map):
         super(TypeQueryTab, self).__init__(parent)
         self.log = logging.getLogger(__name__)
@@ -46,8 +51,10 @@ class TypeQueryTab(AnalysisTab):
         self.setupUi()
 
     def __del__(self):
-        self.thread.quit()
-        self.thread.wait(5000)
+        with suppress(RuntimeError):
+            self.thread.quit()
+            self.thread.wait(5000)
+
         logging.getLogger("setools.typequery").removeHandler(self.handler)
 
     def setupUi(self):
